#include <string.h>
#include <GL/glui.h>

int mainWindow;

float angle = 0;
float dx = 0, dy = 0;
float sx = 1, sy = 1;

int showCoordinateSystem0 = 1;
int showCoordinateSystem1 = 1;
int showCoordinateSystem2 = 1;
int showCoordinateSystem3 = 1;

void idle()
{
	if (glutGetWindow() != mainWindow)
		glutSetWindow(mainWindow);
	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0,0,w,h);
	
	if (w == 0) w = 1;
	if (h == 0) h = 1;

	double aspect = w * 1.0 / h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}

void drawHouse()
{
	glLineWidth(2.0f);
	
	glColor3f(1,1,0);

	glBegin(GL_LINE_LOOP);
	glVertex2f(0,0);
	glVertex2f(2,0);
	glVertex2f(2,2);
	glVertex2f(0,2);
	glEnd();

	glBegin(GL_LINE_STRIP);
	glVertex2f(0,2); 
	glVertex2f(1,3);
	glVertex2f(2,2);
	glEnd();

	glBegin(GL_LINE_STRIP);
	glVertex2f(1.0f/3, 0);
	glVertex2f(1.0f/3, 4.0f/3);
	glVertex2f(3.0f/3, 4.0f/3);
	glVertex2f(3.0f/3, 0);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex2f(1.25f, 0.5f);
	glVertex2f(1.25f, 1);
	glVertex2f(1.75f, 1);
	glVertex2f(1.75f, 0.5f);
	glEnd();
}

float oldLineColor0[] = {0.3f, 0.3f, 0.3f};
float oldXAxisColor0[] = {0.3f, 0.3f, 0.3f};
float oldYAxisColor0[] = {0.3f, 0.3f, 0.3f};

float oldLineColor1[] = {0.6f, 0.3f, 0.6f};
float oldXAxisColor1[] = {0.6f, 0.3f, 0.6f};
float oldYAxisColor1[] = {0.6f, 0.3f, 0.6f};

float oldLineColor2[] = {0.3f, 0.6f, 0.6f};
float oldXAxisColor2[] = {0.3f, 0.6f, 0.6f};
float oldYAxisColor2[] = {0.3f, 0.6f, 0.6f};

float lineColor[] = {0.5f, 0.5f, 1.0f};
float xAxisColor[] = {1.0f, 0.5f, 0.5f};
float yAxisColor[] = {0.5f, 1.0f, 0.5f};

void drawCoordinateSystem(float *lineColor, float *xAxisColor, float *yAxisColor)
{
	glLineWidth(1.0f);

	glColor3fv(lineColor);
	glBegin(GL_LINES);
	for(int x=-20;x<=20;x++)
	{
		glVertex2f(x, -20);
		glVertex2f(x, 20);
	}
	for(int y=-20;y<=20;y++)
	{
		glVertex2f(-20, y);
		glVertex2f(20, y);
	}
	glEnd();

	glPushMatrix();
	glTranslated(-0.1, -0.1, 0);
	glLineWidth(3.0f);
	glColor3fv(xAxisColor);
	glBegin(GL_LINES);
	glVertex2f(0,0);
	glVertex2f(1,0);
	glEnd();

	glLineWidth(3.0f);
	glColor3fv(yAxisColor);
	glBegin(GL_LINES);
	glVertex2f(0,0);
	glVertex2f(0,1);
	glEnd();

	glPopMatrix();
}

void draw()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glScaled(0.2, 0.2, 1);

	if (showCoordinateSystem0)
		drawCoordinateSystem(oldLineColor0, oldXAxisColor0, oldYAxisColor0);

	glTranslated(dx, dy, 0);
	if (showCoordinateSystem1)
		drawCoordinateSystem(oldLineColor1, oldXAxisColor1, oldYAxisColor1);

	glRotated(angle, 0, 0, 1);
	if (showCoordinateSystem2)
		drawCoordinateSystem(oldLineColor2, oldXAxisColor2, oldYAxisColor2);
	
	glScaled(sx, sy, 1);
	if (showCoordinateSystem3)
		drawCoordinateSystem(lineColor, xAxisColor, yAxisColor);
	drawHouse();
}


void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	draw();

	glutSwapBuffers();
}


char translationCommand[256] = "glTranslated(0.00000, 0.00000, 0);";
GLUI_StaticText *translationCommandText;

char scalingCommand[256] = "glScaled(1.00000, 1.00000, 0);";
GLUI_StaticText *scalingCommandText;

char rotationCommand[256] = "glRotated(0.00000, 0, 0, 1);";
GLUI_StaticText *rotationCommandText;

void updateTransformCommandText(int id)
{
	sprintf(rotationCommand, "glRotated(%.3f, 0, 0, 1);", angle);
	rotationCommandText->set_text(rotationCommand);

	sprintf(translationCommand, "glTranslated(%.3f, %.3f, 0);", dx, dy);
	translationCommandText->set_text(translationCommand);

	sprintf(scalingCommand, "glScaled(%.3f, %.3f, 0);", sx, sy);
	scalingCommandText->set_text(scalingCommand);
}

int main(int argc, char **argv)
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);  
	glutInitWindowSize( 600, 600 );

	mainWindow = glutCreateWindow( "Translation Demo" );
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );    

	GLUI *glui = GLUI_Master.create_glui( "Controls" );

	GLUI_Panel *translation_panel = glui->add_panel("Translation");
	translation_panel->set_alignment(GLUI_ALIGN_LEFT);
	translation_panel->set_w(300);
	GLUI_Spinner *translation_x_spinner = glui->add_spinner_to_panel(translation_panel, "x", GLUI_SPINNER_FLOAT, &dx, -1, updateTransformCommandText);
	translation_x_spinner->set_float_limits(-5, 5);
	translation_x_spinner->set_speed(2);
	GLUI_Spinner *translation_y_spinner = glui->add_spinner_to_panel(translation_panel, "y", GLUI_SPINNER_FLOAT, &dy, -1, updateTransformCommandText);
	translation_y_spinner->set_float_limits(-5, 5);
	translation_y_spinner->set_speed(2);

	GLUI_Panel *rotation_panel = glui->add_panel("Rotation");
	rotation_panel->set_alignment(GLUI_ALIGN_LEFT);
	rotation_panel->set_w(300);
	GLUI_Spinner *angle_spinner = glui->add_spinner_to_panel(rotation_panel, "angle", GLUI_SPINNER_FLOAT, &angle, -1, updateTransformCommandText);
	angle_spinner->set_float_limits(-720, 720);
	angle_spinner->set_speed(1);

	GLUI_Panel *scaling_panel = glui->add_panel("Scaling");
	scaling_panel->set_alignment(GLUI_ALIGN_LEFT);
	scaling_panel->set_w(300);
	GLUI_Spinner *scaling_x_spinner = glui->add_spinner_to_panel(scaling_panel, "x", GLUI_SPINNER_FLOAT, &sx, -1, updateTransformCommandText);
	scaling_x_spinner->set_float_limits(-3, 3);
	scaling_x_spinner->set_speed(2);
	GLUI_Spinner *scaling_y_spinner = glui->add_spinner_to_panel(scaling_panel, "y", GLUI_SPINNER_FLOAT, &sy, -1, updateTransformCommandText);
	scaling_y_spinner->set_float_limits(-3, 3);
	scaling_y_spinner->set_speed(2);

	GLUI_Panel *command_panel = glui->add_panel("OpenGL Command");
	command_panel->set_alignment(GLUI_ALIGN_LEFT);
	command_panel->set_w(300);
	translationCommandText = glui->add_statictext_to_panel(command_panel, translationCommand);	
	rotationCommandText = glui->add_statictext_to_panel(command_panel, rotationCommand);	
	scalingCommandText = glui->add_statictext_to_panel(command_panel, scalingCommand);	
	glui->add_statictext_to_panel(command_panel, "drawHouse();");

	GLUI_Panel *display_panel = glui->add_panel("View");
	display_panel->set_alignment(GLUI_ALIGN_LEFT);
	display_panel->set_w(300);
	glui->add_checkbox_to_panel(display_panel, "Show 1st coordinate system", &showCoordinateSystem0);
	glui->add_checkbox_to_panel(display_panel, "Show 2nd coordinate system", &showCoordinateSystem1);
	glui->add_checkbox_to_panel(display_panel, "Show 3rd coordinate system", &showCoordinateSystem2);
	glui->add_checkbox_to_panel(display_panel, "Show 4th coordinate system", &showCoordinateSystem3);

	glui->set_main_gfx_window( mainWindow );

	GLUI_Master.set_glutIdleFunc( idle ); 

	glutMainLoop();

	return EXIT_SUCCESS;
}