#include <string.h>
#include <GL/glui.h>

int mainWindow;

float scaleX = 1, scaleY = 1;

void idle()
{
	if (glutGetWindow() != mainWindow)
		glutSetWindow(mainWindow);
	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0,0,w,h);
	
	if (w == 0) w = 1;
	if (h == 0) h = 1;

	double aspect = w * 1.0 / h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}

void drawHouse()
{
	glLineWidth(2.0f);
	
	glColor3f(1,1,0);

	glBegin(GL_LINE_LOOP);
	glVertex2f(0,0);
	glVertex2f(2,0);
	glVertex2f(2,2);
	glVertex2f(0,2);
	glEnd();

	glBegin(GL_LINE_STRIP);
	glVertex2f(0,2); 
	glVertex2f(1,3);
	glVertex2f(2,2);
	glEnd();

	glBegin(GL_LINE_STRIP);
	glVertex2f(1.0f/3, 0);
	glVertex2f(1.0f/3, 4.0f/3);
	glVertex2f(3.0f/3, 4.0f/3);
	glVertex2f(3.0f/3, 0);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex2f(1.25f, 0.5f);
	glVertex2f(1.25f, 1);
	glVertex2f(1.75f, 1);
	glVertex2f(1.75f, 0.5f);
	glEnd();
}

float lineColor[] = {0.5f, 0.5f, 1.0f};
float xAxisColor[] = {1.0f, 0.5f, 0.5f};
float yAxisColor[] = {0.5f, 1.0f, 0.5f};

void drawCoordinateSystem(float *lineColor, float *xAxisColor, float *yAxisColor)
{
	glLineWidth(1.0f);

	glColor3fv(lineColor);
	glBegin(GL_LINES);
	for(int x=-5;x<=5;x++)
	{
		glVertex2f(x, -5);
		glVertex2f(x, 5);
	}
	for(int y=-5;y<=5;y++)
	{
		glVertex2f(-5, y);
		glVertex2f(5, y);
	}
	glEnd();

	glPushMatrix();
	glTranslated(-0.1, -0.1, 0);
	glLineWidth(3.0f);
	glColor3fv(xAxisColor);
	glBegin(GL_LINES);
	glVertex2f(0,0);
	glVertex2f(1,0);
	glEnd();

	glLineWidth(3.0f);
	glColor3fv(yAxisColor);
	glBegin(GL_LINES);
	glVertex2f(0,0);
	glVertex2f(0,1);
	glEnd();

	glPopMatrix();
}

void draw()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glScaled(0.2, 0.2, 1);

	drawCoordinateSystem(lineColor, xAxisColor, yAxisColor);
	
	glScaled(scaleX, scaleY, 0.0);
	drawHouse();
}


void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	draw();

	glutSwapBuffers();
}

char transformCommand[256] = "glScaled(1.00000, 1.00000, 0);";
GLUI_StaticText *transformCommandText;
GLUI_Spinner *x_spinner, *y_spinner;

void updateTransformCommandText(int id)
{
	sprintf(transformCommand, "glScaled(%.3f, %.3f, 0);", x_spinner->get_float_val(), y_spinner->get_float_val());
	transformCommandText->set_text(transformCommand);
}

int main(int argc, char **argv)
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);  
	glutInitWindowSize( 600, 600 );

	mainWindow = glutCreateWindow( "Translation Demo" );
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );    

	GLUI *glui = GLUI_Master.create_glui( "Controls" );

	GLUI_Panel *scaling_panel = glui->add_panel("Scaling");
	scaling_panel->set_alignment(GLUI_ALIGN_LEFT);
	scaling_panel->set_w(300);
	x_spinner = glui->add_spinner_to_panel(scaling_panel, "x", GLUI_SPINNER_FLOAT, &scaleX, -1, updateTransformCommandText);
	x_spinner->set_float_limits(-3, 3);
	x_spinner->set_speed(2);
	y_spinner = glui->add_spinner_to_panel(scaling_panel, "y", GLUI_SPINNER_FLOAT, &scaleY, -1, updateTransformCommandText);
	y_spinner->set_float_limits(-3, 3);
	y_spinner->set_speed(2);

	GLUI_Panel *command_panel = glui->add_panel("OpenGL Command");
	command_panel->set_alignment(GLUI_ALIGN_LEFT);
	command_panel->set_w(300);
	transformCommandText = glui->add_statictext_to_panel(command_panel, transformCommand);	
	glui->add_statictext_to_panel(command_panel, "drawHouse();");

	glui->set_main_gfx_window( mainWindow );

	GLUI_Master.set_glutIdleFunc( idle ); 

	glutMainLoop();

	return EXIT_SUCCESS;
}